<?php

/**
 * @file
 * Admin page callbacks for the LiftIgniter module.
 */

/**
 * LiftIgniter Configuration form callback.
 */
function liftigniter_configuration_form($form, &$form_state) {
  $form['liftigniter_enable'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable'),
    '#default_value' => variable_get('liftigniter_enable', FALSE),
    '#description' => t('Uncheck this box to completely disable LiftIgniter functionality.'),
  );
  $form['liftigniter_js_access_key'] = array(
    '#title' => t('JS Access Key'),
    '#type' => 'textfield',
    '#required' => TRUE,
    '#description' => t('Enter the JS Key (SDK) of LiftIgniter.</br>JS Key will be found in the !link page of the LiftIgniter console.', array(
      '!link' => l(t('Settings'), "https://console.liftigniter.com/settings/keys", array(
        'attributes' => array(
          'target' => '_blank',
        ),
      )),
    )),
    '#default_value' => variable_get('liftigniter_js_access_key', ''),
  );
  $form['liftigniter_api_access_key'] = array(
    '#type' => 'textfield',
    '#title' => t('API Access Key'),
    '#required' => TRUE,
    '#description' => t('Enter the API Key (REST) of LiftIgniter.</br>API Key will be found in the !link page of the LiftIgniter console.', array(
      '!link' => l(t('Settings'), "https://console.liftigniter.com/settings/keys", array(
        'attributes' => array(
          'target' => '_blank',
        ),
      )),
    )),
    '#default_value' => variable_get('liftigniter_api_access_key', ''),
  );
  $form['liftigniter_request_fields'] = array(
    '#type' => 'textfield',
    '#title' => t('String Value Field List'),
    '#description' => t('Additional fields (String value) to be returned from LiftIgniter.</br>Enter each field name seperated by a comma.</br>If this field is set to "<b>name, title, url</b>" the js code will add <b>$p("setRequestFields",["name","title","url"])</b>'),
    '#default_value' => variable_get('liftigniter_request_fields', ''),
    '#maxlength' => 256,
  );
  $form['liftigniter_array_request_fields'] = array(
    '#type' => 'textfield',
    '#title' => t('Array Value Field List'),
    '#description' => t('Additional fields (Array value) to be returned from LiftIgniter.</br>Enter each field name seperated by a comma.</br>If this field is set to "<b>name, title, url</b>" the js code will add <b>$p("setArrayRequestFields",["name","title","url"])</b>'),
    '#default_value' => variable_get('liftigniter_array_request_fields', ''),
    '#maxlength' => 256,
  );
  $form['liftigniter_set_no_tag'] = array(
    '#type' => 'checkbox',
    '#title' => t('Set No tag'),
    '#default_value' => variable_get('liftigniter_set_no_tag', FALSE),
    '#description' => t("This can be used to configure whether LiftIgniter's tags are added to URLs in items recommended by LiftIgniter. By default, (non-UTM) LiftIgniter tags are added to recommendations. You can disable adding tags to recommendations by enabling this field."),
  );
  $form['liftigniter_config'] = array(
    '#type' => 'textarea',
    '#title' => t('Global Configurations'),
    '#description' => t("Enter the Global Configurations in JSON format. !link to view the complete list of configurations.", array(
      '!link' => l(t('Click here'), "https://liftigniter.readme.io/docs/pinit", array(
        'attributes' => array(
          'target' => '_blank',
        ),
      )),
    )),
    '#default_value' => variable_get('liftigniter_custom_config', '{}'),
    '#element_validate' => array('liftigniter_json_validate'),
  );
  return system_settings_form($form);
}
