<?php

/**
 * @file
 * Include file for LiftIgniter Template.
 */

/**
 * Callback for listing LiftIgniter Templates.
 */
function liftigniter_ui_templates_list() {
  $query = db_select('liftigniter_template', 'liftigniter_template')->extend('PagerDefault');
  $query->fields('liftigniter_template');
  $query->limit(20);
  $templates = $query->execute()->fetchAll();
  $header = array(
    'title' => t('Title'),
    'id' => t('ID'),
    'status' => t('Status'),
    'operation' => t('Operations'),
  );
  $rows = [];
  if ($templates) {
    foreach ($templates as $template) {
      if (module_exists('ctools')) {
        $links = array(
          array('title' => t('Edit'), 'href' => 'admin/config/services/liftigniter/template/' . $template->id . '/edit'),
          array('title' => t('Delete'), 'href' => 'admin/config/services/liftigniter/template/' . $template->id . '/delete'),
        );
        $action_link = theme('links__ctools_dropbutton', array('links' => $links));
      }
      else {
        $links = array(
          l(t('Edit'), 'admin/config/services/liftigniter/template/' . $template->id . '/edit'),
          l(t('Delete'), 'admin/config/services/liftigniter/template/' . $template->id . '/delete'),
        );
        $action_link = theme('item_list', array(
          'items' => $links,
          'attributes' => array(
            'class' => array(
              'links',
              'inline',
            ),
          ),
        ));
      }
      $rows[] = array(
        'title' => $template->title,
        'id' => $template->id,
        'status' => $template->status ? t('Published') : t('Unpublished'),
        'operation' => $action_link,
      );
    }
    if (module_exists('ctools') && module_exists('views_ui')) {
      // Add ctools dropdown css from views ui module.
      if (module_load_include('inc', 'views_ui', 'includes/admin')) {
        foreach (views_ui_get_admin_css() as $file => $options) {
          drupal_add_css($file, $options);
        }
      }
    }
  }
  return theme('table', array(
    'header' => $header,
    'rows' => $rows,
    'empty' => t('No template available.'),
  ));
}

/**
 * Callback for Createing/Editing LiftIgniter Template.
 */
function liftigniter_ui_template_create($form, &$form_state, $liftigniter_template = NULL) {
  $form = array();
  $defaults = array(
    'id' => '',
    'title' => '',
    'twig' => '',
    'status' => TRUE,
  );
  if ($liftigniter_template) {
    $defaults = array(
      'id' => $liftigniter_template['id'],
      'title' => $liftigniter_template['title'],
      'twig' => $liftigniter_template['twig'],
      'status' => $liftigniter_template['status'],
    );
  }
  $form = array(
    'title' => array(
      '#type' => 'textfield',
      '#title' => t('Title'),
      '#description' => t('Title of the template.'),
      '#required' => TRUE,
      '#default_value' => $defaults['title'],
      '#attributes' => array('maxlength' => 255),
    ),
    'id' => array(
      '#type' => 'machine_name',
      '#title' => t('Template Machine name'),
      '#description' => t('A unique machine name of the template. It must only contain lowercase letters, numbers and underscore.'),
      '#default_value' => $defaults['id'],
      '#maxlength' => 120,
      '#machine_name' => array(
        'exists' => 'liftigniter_ui_template_load',
        'source' => array('title'),
      ),
      '#disabled' => !empty($defaults['id']),
    ),
    'twig' => array(
      '#type' => 'textarea',
      '#title' => t('JS Twig Template'),
      '#description' => t('LiftIgniter JS Twig Template.'),
      '#required' => TRUE,
      '#default_value' => $defaults['twig'],
    ),
    'status' => array(
      '#type' => 'checkbox',
      '#title' => t('Published'),
      '#description' => t('Status of the template'),
      '#default_value' => $defaults['status'],
    ),
    'submit' => array(
      '#type' => 'submit',
      '#value' => t('Save'),
    ),
  );
  return $form;
}

/**
 * Validation handler for creating and updating the LiftIgniter template.
 */
function liftigniter_ui_template_create_validate($form, &$form_state) {
  // TODO: Add validation of Twig Template.
}

/**
 * Submit handler for creating and updating the LiftIgniter template.
 */
function liftigniter_ui_template_create_submit($form, &$form_state) {
  db_merge('liftigniter_template')
    ->key(array('id' => $form_state['values']['id']))
    ->fields(array(
      'title' => $form_state['values']['title'],
      'twig' => $form_state['values']['twig'],
      'status' => $form_state['values']['status'],
    ))
    ->execute();
  $form_state['redirect'] = 'admin/config/services/liftigniter/template';
  drupal_set_message(
    t("The %title template has been Added/Updated.",
      array(
        '%title' => $form_state['values']['title'],
      )
    )
  );
}

/**
 * Callback for deleting LiftIgniter Template.
 */
function liftigniter_ui_template_delete($form, &$form_state, $liftigniter_template) {
  $form = array(
    'id'  => array(
      '#type' => 'hidden',
      '#value' => $liftigniter_template['id'],
    ),
    'title'  => array(
      '#type' => 'hidden',
      '#value' => $liftigniter_template['title'],
    ),
  );
  return confirm_form(
    $form,
    t('Are you sure you want to delete %title?',
      array(
        '%title' => $liftigniter_template['title'],
      )
    ),
    'admin/config/services/liftigniter/template',
    NULL,
    t('Delete'),
    t('Cancel')
  );
}

/**
 * Submit handler for delete confirm form.
 */
function liftigniter_ui_template_delete_submit($form, &$form_state) {
  db_delete('liftigniter_template')
    ->condition('id', $form_state['values']['id'])
    ->execute();
  drupal_set_message(
    t("The %title template has been deleted.",
      array(
        '%title' => $form_state['values']['title'],
      )
    )
  );
  $form_state['redirect'] = 'admin/config/services/liftigniter/template';
}

/**
 * LiftIgniter Template Title Callback for Menu.
 */
function liftigniter_ui_template_edit_title($liftigniter_template) {
  return t('Edit Template @title', array('@title' => $liftigniter_template['title']));
}
