<?php

/**
 * @file
 * Include file for LiftIgniter Widget.
 */

/**
 * Callback for listing LiftIgniter Widgets.
 */
function liftigniter_ui_widgets_list() {
  $query = db_select('liftigniter_widget', 'liftigniter_widget')->extend('PagerDefault');
  $query->fields('liftigniter_widget');
  $query->limit(20);
  $widgets = $query->execute()->fetchAll();
  $header = array(
    'title' => t('Title'),
    'id' => t('ID'),
    'status' => t('Status'),
    'operation' => t('Operations'),
  );
  $rows = [];
  if ($widgets) {
    foreach ($widgets as $widget) {
      if (module_exists('ctools')) {
        $links = array(
          array('title' => t('Edit'), 'href' => 'admin/config/services/liftigniter/widget/' . $widget->id . '/edit'),
          array('title' => t('Delete'), 'href' => 'admin/config/services/liftigniter/widget/' . $widget->id . '/delete'),
        );
        $action_link = theme('links__ctools_dropbutton', array('links' => $links));
      }
      else {
        $links = array(
          l(t('Edit'), 'admin/config/services/liftigniter/widget/' . $widget->id . '/edit'),
          l(t('Delete'), 'admin/config/services/liftigniter/widget/' . $widget->id . '/delete'),
        );
        $action_link = theme('item_list', array(
          'items' => $links,
          'attributes' => array(
            'class' => array(
              'links',
              'inline',
            ),
          ),
        ));
      }
      $rows[] = array(
        'title' => $widget->title,
        'id' => $widget->id,
        'status' => $widget->status ? t('Published') : t('Unpublished'),
        'operation' => $action_link,
      );
    }
    if (module_exists('ctools') && module_exists('views_ui')) {
      // Add ctools dropdown css from views ui module.
      if (module_load_include('inc', 'views_ui', 'includes/admin')) {
        foreach (views_ui_get_admin_css() as $file => $options) {
          drupal_add_css($file, $options);
        }
      }
    }
  }
  return theme('table', array(
    'header' => $header,
    'rows' => $rows,
    'empty' => t('No widget available.'),
  ));
}

/**
 * Callback for Createing/Editing LiftIgniter Widget.
 */
function liftigniter_ui_widget_create($form, &$form_state, $liftigniter_widget = NULL) {
  $templates = liftigniter_template_load_multiple(TRUE);
  $template_options = array();
  if (!$liftigniter_widget && !$templates) {
    $form['no_result']['#markup'] = t('You should have atleast one published template to create a widget.');
    return $form;
  }
  $template_options = array_map(function ($template) {
    return $template['title'];
  }, $templates);
  $form = array();
  $defaults = array(
    'id' => '',
    'title' => '',
    'settings' => array(
      'widget_details' => array(
        'widget' => '',
        'selector' => '',
        'no_of_items' => 'dynamic',
        'max' => '',
        'opts' => '{}',
      ),
      'ab_testing' => array(
        'enable' => FALSE,
        'slice' => '',
      ),
      'renderer' => array(
        'method' => 'all',
        'selector' => '',
        'template' => '',
      ),
      'track' => array(
        'li' => array(
          'enable' => FALSE,
          'selector' => '',
          'method' => 'anchor',
          'tag_prefix' => '',
          'debug' => FALSE,
        ),
        'base' => array(
          'enable' => FALSE,
          'selector' => '',
          'method' => 'anchor',
          'tag_prefix' => '',
          'debug' => FALSE,
        ),
      ),
    ),
    'status' => TRUE,
  );
  if ($liftigniter_widget) {
    $defaults = array(
      'id' => $liftigniter_widget['id'],
      'title' => $liftigniter_widget['title'],
      'settings' => $liftigniter_widget['settings'],
      'status' => $liftigniter_widget['status'],
    );
  }
  $form = array(
    'title' => array(
      '#type' => 'textfield',
      '#title' => t('Title'),
      '#description' => t('Title of the widget.'),
      '#required' => TRUE,
      '#default_value' => $defaults['title'],
      '#attributes' => array('maxlength' => 255),
    ),
    'id' => array(
      '#type' => 'machine_name',
      '#title' => t('Widget Machine name'),
      '#description' => t('A unique machine name of the widget. It must only contain lowercase letters, numbers and hyphens.'),
      '#default_value' => $defaults['id'],
      '#maxlength' => 120,
      '#machine_name' => array(
        'exists' => 'liftigniter_ui_widget_load',
        'source' => array('title'),
      ),
      '#disabled' => !empty($defaults['id']),
    ),
  );
  $form['vertical_tabs'] = [
    '#type' => 'vertical_tabs',
  ];
  $form['widget_details'] = [
    '#type' => 'fieldset',
    '#title' => t('Details'),
    '#tree' => TRUE,
    '#group' => 'vertical_tabs',
  ];
  $form['widget_details']['widget'] = [
    '#type' => 'textfield',
    '#title' => t("Widget"),
    '#default_value' => $defaults['settings']['widget_details']['widget'],
    '#description' => t("Widget to register in LiftIgniter"),
    '#required' => TRUE,
  ];
  $form['widget_details']['selector'] = [
    '#type' => 'textfield',
    '#title' => t("Selector"),
    '#default_value' => $defaults['settings']['widget_details']['selector'],
    '#description' => t("Selector for the widget"),
    '#required' => TRUE,
  ];
  $form['widget_details']['no_of_items'] = [
    '#type' => 'radios',
    '#title' => t("No. of Items."),
    '#options' => [
      'dynamic' => t("Dynamic (Use Element length as Max)"),
      'static' => t("Static"),
    ],
    '#default_value' => $defaults['settings']['widget_details']['no_of_items'],
    '#description' => t("Number of Items to be requested by the widget"),
    '#required' => TRUE,
  ];
  $form['widget_details']['max'] = [
    '#type' => 'textfield',
    '#element_validate' => array('element_validate_integer_positive'),
    '#title' => t('Maximum no. of items'),
    '#min' => 1,
    '#default_value' => $defaults['settings']['widget_details']['max'],
    '#description' => t('Enter Maximum no. of items to be requested by this Widget.'),
    '#states' => [
      'visible' => [
        'input[name="widget_details[no_of_items]"]' => [
          'value' => 'static',
        ],
      ],
      'required' => [
        'input[name="widget_details[no_of_items]"]' => [
          'value' => 'static',
        ],
      ],
    ],
  ];
  $form['widget_details']['opts'] = [
    '#type' => 'textarea',
    '#title' => t('Custom Options for the widget'),
    '#default_value' => $defaults['settings']['widget_details']['opts'],
    '#description' => t('Enter custom options (opts) for this Widget in Json Format.'),
    '#element_validate' => array('liftigniter_json_validate'),
  ];
  $form['ab_testing'] = [
    '#type' => 'fieldset',
    '#title' => t('AB Testing'),
    '#tree' => TRUE,
    '#group' => 'vertical_tabs',
  ];
  $form['ab_testing']['enable'] = [
    '#type' => 'checkbox',
    '#title' => t('Enable'),
    '#default_value' => $defaults['settings']['ab_testing']['enable'],
    '#description' => t('Enabling will add AB testing for this widget.'),
  ];
  $form['ab_testing']['slice'] = [
    '#type' => 'textfield',
    '#element_validate' => array('element_validate_integer_positive'),
    '#title' => t('Slice'),
    '#default_value' => $defaults['settings']['ab_testing']['slice'],
    '#description' => t('Enter the slice value for AB Testing.'),
    '#states' => [
      'visible' => [
        'input[name="ab_testing[enable]"]' => [
          'checked' => TRUE,
        ],
      ],
      'required' => [
        'input[name="ab_testing[enable]"]' => [
          'checked' => TRUE,
        ],
      ],
    ],
  ];
  $form['renderer'] = [
    '#type' => 'fieldset',
    '#title' => t('Render'),
    '#tree' => TRUE,
    '#group' => 'vertical_tabs',
  ];
  $form['renderer']['method'] = [
    '#type' => 'radios',
    '#title' => t('Rendering Method'),
    '#options' => [
      'all' => t("Render all Inventories"),
      'each' => t("Render Inventories one by one"),
    ],
    '#default_value' => $defaults['settings']['renderer']['method'],
    '#description' => t('Choose the method to render this widget.'),
    '#required' => TRUE,
  ];
  $form['renderer']['selector'] = [
    '#type' => 'textfield',
    '#title' => t('Selector'),
    '#default_value' => $defaults['settings']['renderer']['selector'],
    '#description' => t('Enter the selector to render this widget.'),
    '#required' => TRUE,
  ];
  $form['renderer']['template'] = [
    '#type' => 'select',
    '#title' => t('Template'),
    '#options' => $template_options,
    '#default_value' => $defaults['settings']['renderer']['template'],
    '#description' => t('Choose template to render this widget.'),
    '#required' => TRUE,
  ];

  // Tracking.
  $form['track'] = [
    '#type' => 'fieldset',
    '#title' => t('Tracking'),
    '#tree' => TRUE,
    '#group' => 'vertical_tabs',
  ];
  foreach (['li' => 'LI', 'base' => 'Base'] as $key => $value) {
    $form['track'][$key] = [
      '#type' => 'fieldset',
      '#title' => t(':value Tracking', [':value' => $value]),
    ];
    $form['track'][$key]['enable'] = [
      '#type' => 'checkbox',
      '#title' => t('Enable :value', [':value' => $value]),
      '#default_value' => $defaults['settings']['track'][$key]['enable'],
      '#description' => t('Enable :value tracking.', [':value' => $value]),
    ];
    $form['track'][$key]['method'] = [
      '#type' => 'radios',
      '#title' => t('Tracking Method'),
      '#options' => [
        'anchor' => t("Anchor Tracking"),
        'non_anchor' => t("Non-Anchor Tracking"),
      ],
      '#default_value' => $defaults['settings']['track'][$key]['method'],
      '#description' => t('Choose the method for tracking this widget.'),
      '#states' => [
        'visible' => [
          'input[name="track[' . $key . '][enable]"]' => [
            'checked' => TRUE,
          ],
        ],
        'required' => [
          'input[name="track[' . $key . '][enable]"]' => [
            'checked' => TRUE,
          ],
        ],
      ],
    ];
    $form['track'][$key]['selector'] = [
      '#type' => 'textfield',
      '#title' => t('Selector'),
      '#default_value' => $defaults['settings']['track'][$key]['selector'],
      '#description' => t('Enter the selector for tracking this widget.'),
      '#states' => [
        'visible' => [
          'input[name="track[' . $key . '][enable]"]' => [
            'checked' => TRUE,
          ],
        ],
        'required' => [
          'input[name="track[' . $key . '][enable]"]' => [
            'checked' => TRUE,
          ],
        ],
      ],
    ];
    $form['track'][$key]['tag_prefix'] = [
      '#type' => 'checkbox',
      '#title' => t('Tag Prefix'),
      '#default_value' => $defaults['settings']['track'][$key]['tag_prefix'],
      '#description' => t("Tag Prefix changes the prefix of URL tag that's appended to href of anchor being tracked. Default to 'li' like li_medium=WIDGET_NAME&li_source=LI. However, with the Tag Prefix 'utm', this will change to the following: utm_medium=WIDGET_NAME&utm_source=LI"),
      '#states' => [
        'visible' => [
          'input[name="track[' . $key . '][enable]"]' => [
            'checked' => TRUE,
          ],
          'input[name="track[' . $key . '][method]"]' => [
            'value' => 'anchor',
          ],
        ],
      ],
    ];
    $form['track'][$key]['debug'] = [
      '#type' => 'checkbox',
      '#title' => t('Debug'),
      '#default_value' => $defaults['settings']['track'][$key]['debug'],
      '#description' => t('Enabling will debug anchor tracking for this widget.'),
      '#states' => [
        'visible' => [
          'input[name="track[' . $key . '][enable]"]' => [
            'checked' => TRUE,
          ],
          'input[name="track[' . $key . '][method]"]' => [
            'value' => 'anchor',
          ],
        ],
      ],
    ];
  }
  $form['status'] = array(
    '#type' => 'checkbox',
    '#title' => t('Published'),
    '#description' => t('Status of the widget'),
    '#default_value' => $defaults['status'],
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save Widget'),
  );
  return $form;
}

/**
 * Validation handler for creating and updating the LiftIgniter widget.
 */
function liftigniter_ui_widget_create_validate($form, &$form_state) {
  if ($form_state['values']['ab_testing']['enable']) {
    if (empty($form_state['values']['ab_testing']['slice'])) {
      form_set_error('ab_testing][slice', t('AB Testing Slice value cannot be empty.'));
    }
    elseif ($form_state['values']['ab_testing']['slice'] < 1 || $form_state['values']['ab_testing']['slice'] > 100) {
      form_set_error('ab_testing][slice', t('AB Testing Slice value should be between 1 to 100.'));
    }
  }
  foreach (array('li' => 'LI', 'base' => 'Base') as $track => $track_label) {
    if ($form_state['values']['track'][$track]['enable']) {
      if (empty($form_state['values']['track'][$track]['method'])) {
        form_set_error('track][' . $track . '][method', t('Choose a Tracking Method for :track tracking.', array(':track' => $track_label)));
      }
      if (empty($form_state['values']['track'][$track]['selector'])) {
        form_set_error('track][' . $track . '][selector', t('Selector cannot be empty for :track tracking.', array(':track' => $track_label)));
      }
    }
  }
}

/**
 * Submit handler for creating and updating the LiftIgniter widget.
 */
function liftigniter_ui_widget_create_submit($form, &$form_state) {
  $settings = array(
    'widget_details' => $form_state['values']['widget_details'],
    'ab_testing' => $form_state['values']['ab_testing'],
    'renderer' => $form_state['values']['renderer'],
    'track' => $form_state['values']['track'],
  );
  db_merge('liftigniter_widget')
    ->key(array('id' => $form_state['values']['id']))
    ->fields(array(
      'title' => $form_state['values']['title'],
      'settings' => serialize($settings),
      'status' => $form_state['values']['status'],
    ))
    ->execute();
  $form_state['redirect'] = 'admin/config/services/liftigniter/widget';
  drupal_set_message(
    t("The %title widget has been Added/Updated.",
      array(
        '%title' => $form_state['values']['title'],
      )
    )
  );
}

/**
 * Callback for deleting LiftIgniter Widget.
 */
function liftigniter_ui_widget_delete($form, &$form_state, $liftigniter_widget) {
  $form = array(
    'id'  => array(
      '#type' => 'hidden',
      '#value' => $liftigniter_widget['id'],
    ),
    'title'  => array(
      '#type' => 'hidden',
      '#value' => $liftigniter_widget['title'],
    ),
  );
  return confirm_form(
    $form,
    t('Are you sure you want to delete %title?',
      array(
        '%title' => $liftigniter_widget['title'],
      )
    ),
    'admin/config/services/liftigniter/widget',
    NULL,
    t('Delete'),
    t('Cancel')
  );
}

/**
 * Submit handler for delete confirm form.
 */
function liftigniter_ui_widget_delete_submit($form, &$form_state) {
  db_delete('liftigniter_widget')
    ->condition('id', $form_state['values']['id'])
    ->execute();
  drupal_set_message(
    t("The %title widget has been deleted.",
      array(
        '%title' => $form_state['values']['title'],
      )
    )
  );
  $form_state['redirect'] = 'admin/config/services/liftigniter/widget';
}

/**
 * LiftIgniter Widget Title Callback for Menu.
 */
function liftigniter_ui_widget_edit_title($liftigniter_widget) {
  return t('Edit Widget @title', array('@title' => $liftigniter_widget['title']));
}
